/*
[Core]
// No data kickoff delay from subspace
PacketSilenceLimit=2000
// Higher numbers mean faster uploads, but more ploss
MaxReliableInTransit=3
// Decrease this number if you want to minimize b/w
ChunkSize=496
// PriitK's directories aren't compatible
ClusterMode=0
// Maximum number of simultaneous uploads
MaxHosts=1

[CPU]
// Set high if you see this warning constantly
SlowIteration=300
// CPU hogging is not good
IterationSleepTime=15

[Zone]
// Delay between initial pings
SweepTime=500
// Maximum number of pings dropped
DropLimit=10
// Time until a zone's password is reclaimed in hours
ReclaimDelay=24
// Initial time used to discover a dropped ping
InitialHighPing=300

[Directory]
// Trusted hosts to download the server list from
IP=sscentral.ds98.com, sscentral.subspace.net & sscentral.subspace.inet.fi
// Time between consecutive downloads in minutes
DownloadTime=1
// Stats file
StatsFile=stats.txt
// Time between writes in minutes
StatsWriteMinutes=1
*/

#ifndef SETTINGS_H
#define SETTINGS_H

#include "datatypes.h"
#include "fileio.h"
#include "sockets.h"


// Directory settings

#define INI_NAME "ddirserv.ini"

Uint32 getPrivateProfileLong(char * section, char * key, char * def, char * path);

struct directorySettings
{
	// [Core]
	Uint32 PacketSilenceLimit;
	Uint32 MaxReliableInTransit;
	Uint32 ChunkSize;
	bool ClusterMode;
	Uint32 MaxHosts;

	// [CPU]
	Uint32 SlowIteration;
	Uint32 IterationSleepTime;

	// [Zone]
	Uint32 SweepTime;
	Uint32 DropLimit;
	Uint32 ReclaimDelay;
	Uint32 InitialHighPing;

	// [Directory]
	Uint32 DownloadTime;
	Uint32 TrustedHosts[10];
	Uint32 TotalHosts;
	char StatsFile[32];
	Uint32 StatsWriteMinutes;

	directorySettings();
	~directorySettings();

	void deleteStrings();

	void reloadSettings();
};


// Ship settings

struct shipSettings					// 144 bytes wide
{
	Uint16 SuperTime;
	Uint16 UNKNOWN0;				// ?
	Uint16 ShieldsTime;
	Uint16 UNKNOWN1;				// ?
	Uint16 Gravity;
	Uint16 GravityTopSpeed;
	Uint16 BulletFireEnergy;
	Uint16 MultiFireEnergy;
	Uint16 BombFireEnergy;
	Uint16 BombFireEnergyUpgrade;
	Uint16 LandmineFireEnergy;
	Uint16 LandmineFireEnergyUpgrade;
	Uint16 BulletSpeed;
	Uint16 BombSpeed;
	Uint16 UNKNOWN2;				// ?
	Uint16 MultiFireAngle;
	Uint16 CloakEnergy;
	Uint16 StealthEnergy;
	Uint16 AntiWarpEnergy;
	Uint16 XRadarEnergy;
	Uint16 MaximumRotation;
	Uint16 MaximumThrust;
	Uint16 MaximumSpeed;
	Uint16 MaximumRecharge;
	Uint16 MaximumEnergy;
	Uint16 InitialRotation;
	Uint16 InitialThrust;
	Uint16 InitialSpeed;
	Uint16 InitialRecharge;
	Uint16 InitialEnergy;
	Uint16 UpgradeRotation;
	Uint16 UpgradeThrust;
	Uint16 UpgradeSpeed;
	Uint16 UpgradeRecharge;
	Uint16 UpgradeEnergy;
	Uint16 AfterburnerEnergy;
	Uint16 BombThrust;
	Uint16 BurstSpeed;
	Uint16 TurretThrustPenalty;
	Uint16 TurretSpeedPenalty;
	Uint16 BulletFireDelay;
	Uint16 MultiFireDelay;
	Uint16 BombFireDelay;
	Uint16 LandmineFireDelay;
	Uint16 RocketTime;
	Uint16 InitialBounty;
	Uint16 DamageFactor;
	Uint16 PrizeShareLimit;
	Uint16 AttachBounty;
	Uint16 SoccerThrowTime;
	Uint16 SoccerBallFriction;
	Uint16 SoccerBallProximity;
	Uint16 SoccerBallSpeed;

	BYTE TurretLimit;
	BYTE BurstShrapnel;
	BYTE MaxMines;
	BYTE RepelMax;
	BYTE BurstMax;
	BYTE DecoyMax;
	BYTE ThorMax;
	BYTE BrickMax;
	BYTE RocketMax;
	BYTE PortalMax;
	BYTE InitialRepel;
	BYTE InitialBurst;
	BYTE InitialBrick;
	BYTE InitialRocket;
	BYTE InitialThor;
	BYTE InitialDecoy;
	BYTE InitialPortal;
	BYTE BombBounceCount;

	unsigned ShrapnelMax	: 4;
	unsigned ShrapnelRate	: 4;

	unsigned SuperStatus	: 2;
	unsigned CloakStatus	: 2;
	unsigned StealthStatus	: 2;
	unsigned XRadarStatus	: 2;

	unsigned AntiwarpStatus	: 2;
	unsigned InitialGuns	: 2;
	unsigned MaxGuns		: 2;
	unsigned InitialBombs	: 2;

	unsigned MaxBombs		: 2;
	unsigned DoubleBarrel	: 1;
	unsigned EmpBomb		: 1;
	unsigned SeeMines		: 1;
	unsigned UNKNOWN3		: 3;	// ?

	BYTE UNKNOWN4[16];				// ?
};


// Prize settings

struct prizeSettings				// 28 bytes wide
{
	BYTE QuickCharge;
	BYTE Energy;
	BYTE Rotation;
	BYTE Stealth;
	BYTE Cloak;
	BYTE XRadar;
	BYTE Warp;
	BYTE Gun;
	BYTE Bomb;
	BYTE BouncingBullets;
	BYTE Thruster;
	BYTE TopSpeed;
	BYTE Recharge;
	BYTE Glue;
	BYTE MultiFire;
	BYTE Proximity;
	BYTE AllWeapons;
	BYTE Shields;
	BYTE Shrapnel;
	BYTE AntiWarp;
	BYTE Repel;
	BYTE Burst;
	BYTE Decoy;
	BYTE Thor;
	BYTE MultiPrize;
	BYTE Brick;
	BYTE Rocket;
	BYTE Portal;
};


// Arena settings

struct arenaSettings				// 1428 bytes wide
{
	Uint32 ident;					// 0000 Constantly 0F 00 00 00
	shipSettings ships[8];			// 0004
	Uint32 BulletDamageLevel;		// 1156 BulletDamageLevel * 1000
	Uint32 BombDamageLevel;			// 1160 BombDamageLevel * 1000
	Uint32 BulletAliveTime;			// 1164
	Uint32 BombAliveTime;			// 1168
	Uint32 DecoyAliveTime;			// 1172
	Uint32 SafetyLimit;				// 1176
	Uint32 FrequencyShift;			// 1180
	Uint32 MaxFrequency;			// 1184
	Uint32 RepelSpeed;				// 1188
	Uint32 MineAliveTime;			// 1192
	Uint32 BurstDamageLevel;		// 1196 BurstDamageLevel * 1000
	Uint32 BulletDamageUpgrade;		// 1200 BulletDamageUpgrade * 1000
	Uint32 FlagDropDelay;			// 1204
	Uint32 EnterGameFlaggingDelay;	// 1208
	Uint32 RocketThrust;			// 1212
	Uint32 RocketSpeed;				// 1216
	Uint32 InactiveShrapnelDamage;	// 1220 ? InactiveShrapnelDamage * 1000
	Uint32 WormholeSwitchTime;		// 1224
	Uint32 ActivateAppShutdownTime;	// 1228
	Uint32 ShrapnelSpeed;			// 1232
	BYTE UNKNOWN0[16];				// 1236 ?
	Uint16 SendRoutePercent;		// 1252
	Uint16 BombExplodeDelay;		// 1254
	Uint16 SendPositionDelay;		// 1256
	Uint16 BombExplodePixels;		// 1258
	Uint16 DeathPrizeTime;			// 1260
	Uint16 JitterTime;				// 1262
	Uint16 EnterDelay;				// 1264
	Uint16 EngineShutdownTime;		// 1266
	Uint16 ProximityDistance;		// 1268
	Uint16 BountyIncreaseForKill;	// 1270
	Uint16 BounceFactor;			// 1272
	Uint16 MapZoomFactor;			// 1274
	Uint16 MaxBonus;				// 1276
	Uint16 MaxPenalty;				// 1278
	Uint16 RewardBase;				// 1280
	Uint16 RepelTime;				// 1282
	Uint16 RepelDistance;			// 1284
	Uint16 HelpTickerDelay;			// 1286
	Uint16 FlaggerOnRadar;			// 1288
	Uint16 FlaggerKillMultiplier;	// 1290
	Uint16 PrizeFactor;				// 1292
	Uint16 PrizeDelay;				// 1294
	Uint16 PrizeMinimumVirtual;		// 1296
	Uint16 PrizeUpgradeVirtual;		// 1298
	Uint16 PrizeMaxExist;			// 1300
	Uint16 PrizeMinExist;			// 1302
	Uint16 PrizeNegativeFactor;		// 1304
	Uint16 DoorDelay;				// 1306
	Uint16 AntiwarpPixels;			// 1308
	Uint16 DoorMode;				// 1310
	Uint16 FlagBlankDelay;			// 1312
	Uint16 NoDataFlagDropDelay;		// 1314
	Uint16 MultiPrizeCount;			// 1316
	Uint16 BrickTime;				// 1318
	Uint16 WarpRadiusLimit;			// 1320
	Uint16 EBombShutdownTime;		// 1322
	Uint16 EBombDamagePercent;		// 1324
	Uint16 RadarNeutralSize;		// 1326
	Uint16 WarpPointDelay;			// 1328
	Uint16 NearDeathLevel;			// 1330
	Uint16 BBombDamagePercent;		// 1332
	Uint16 ShrapnelDamagePercent;	// 1334
	Uint16 ClientSlowPacketTime;	// 1336
	Uint16 FlagDropResetReward;		// 1338
	Uint16 FlaggerFireCostPercent;	// 1340
	Uint16 FlaggerDamagePercent;	// 1342
	Uint16 FlaggerBombFireDelay;	// 1344
	Uint16 SoccerPassDelay;			// 1346
	Uint16 SoccerBallBlankDelay;	// 1348
	Uint16 S2CNoDataKickoutDelay;	// 1350
	Uint16 FlaggerThrustAdjustment;	// 1352
	Uint16 FlaggerSpeedAdjustment;	// 1354
	Uint16 CliSlowPacketSampleSize;	// 1356
	BYTE UNKNOWN1[10];				// 1358 ?
	BYTE RandomShrapnel;			// 1368
	BYTE SoccerBallBounce;			// 1369
	BYTE SoccerAllowBombs;			// 1370
	BYTE SoccerAllowGuns;			// 1371
	BYTE SoccerMode;				// 1372
	BYTE MaxPerTeam;				// 1373
	BYTE MaxPerPrivateTeam;			// 1374
	BYTE TeamMaxMines;				// 1375
	BYTE WormholeGravityBombs;		// 1376
	BYTE BombSafety;				// 1377
	BYTE MessageReliable;			// 1378
	BYTE TakePrizeReliable;			// 1379
	BYTE AllowAudioMessages;		// 1380
	BYTE PrizeHideCount;			// 1381
	BYTE ExtraPositionData;			// 1382
	BYTE SlowFrameCheck;			// 1383
	BYTE CarryFlags;				// 1384
	BYTE AllowSavedShip;			// 1385
	BYTE RadarMode;					// 1386
	BYTE VictoryMusic;				// 1387
	BYTE FlaggerGunUpgrade;			// 1388
	BYTE FlaggerBombUpgrade;		// 1389
	BYTE SoccerUserFlagger;			// 1390
	BYTE SoccerBallLocation;		// 1391
	BYTE UNKNOWN2[8];				// 1392 ?
	prizeSettings pw;				// 1400
};

#endif	// SETTINGS_H
