#include "sockets.h"


//////// Logging ////////

void WriteLog(char * Packet, int Length)
{
	char stringRep[17];
	memset(stringRep, 0, 17);
	int strOffset = 0;
	char t;

	printf("s2c 0000  ");

	for (int cnt = 0; cnt < Length; cnt++)
	{
		if (cnt && ((cnt & 15) == 0))
		{
			printf("  %s\ns2c %04X  ", stringRep, cnt);

			memset(stringRep, 0, 17);
			strOffset = 0;
		}
		printf("%02X ", (BYTE)Packet[cnt]);

		t = Packet[cnt];
		if ((t >= 32) && (t <= 126))
			stringRep[strOffset++] = Packet[cnt];
		else
			stringRep[strOffset++] = '.';
	}
	while ((cnt & 15) != 0)
	{
		printf("   ");
		cnt++;
	}
	printf("  %s\n\n", stringRep);
}

void ReadLog(char * Packet, int Length)
{
	char stringRep[17];
	memset(stringRep, 0, 17);
	int strOffset = 0;
	char t;

	printf("c2s 0000  ");

	for (int cnt = 0; cnt < Length; cnt++)
	{
		if (cnt && ((cnt & 15) == 0))
		{
			printf("  %s\nc2s %04X  ", stringRep, cnt);

			memset(stringRep, 0, 17);
			strOffset = 0;
		}
		printf("%02X ", (BYTE)Packet[cnt]);

		t = Packet[cnt];
		if ((t >= 32) && (t <= 126))
			stringRep[strOffset++] = Packet[cnt];
		else
			stringRep[strOffset++] = '.';
	}
	while ((cnt & 15) != 0)
	{
		printf("   ");
		cnt++;
	}
	printf("  %s\n\n", stringRep);
}


//////// Packets ////////

UDPPacket::UDPPacket(INADDR src, BYTE * msg, int len)
{
	this->src = src;
	this->len = len;
	if (len > 0)
	{
		this->msg = new char[len];
		memcpy(this->msg, msg, len);
	}
	else
	{
		this->msg = NULL;
	}
}

UDPPacket::~UDPPacket()
{
	if (msg)
		delete []msg;
}


//////// Winsock2 interface ////////

void beginWinsock()
{
	WSADATA wsaData;
	WSAStartup(0x202, &wsaData);
}

void endWinsock()
{
	WSACleanup();
}


//////// UDP Sockets ////////

UDPSocket::UDPSocket()
{
	sid = SOCKET_ERROR;
}

UDPSocket::~UDPSocket()
{
	if (sid != SOCKET_ERROR)
	{
		WSACloseEvent(event);
		closesocket(sid);
	}
}

int UDPSocket::create(WORD port)
{
	if (sid != SOCKET_ERROR)
	{
		WSACloseEvent(event);
		closesocket(sid);
	}

	sid = socket(AF_INET, PF_INET, IPPROTO_IP);

	if (sid == SOCKET_ERROR)
	{
		printf("socket(AF_INET, PF_INET, IPPROTO_IP) error\n");
		return SOCKET_ERROR;
	}

	INADDR sa(0, port);

	if (bind(sid, sa.getAddress(), sizeof(INADDR)) == SOCKET_ERROR)
	{
		printf("bind(%i) error\n", port);
		return SOCKET_ERROR;
	}

	event = WSACreateEvent();
	WSAEventSelect(sid, event, FD_READ);

	return sid;
}

void UDPSocket::set(DWORD ip, WORD port)
{
	remote.set(ip, port);
}

void UDPSocket::set(INADDR host)
{
	remote = host;
}

bool UDPSocket::send(BYTE * msg, int len)
{
	return sendto(sid, (char*)msg, len, 0, remote.getAddress(), sizeof(INADDR)) != SOCKET_ERROR;
}

bool UDPSocket::send(char * msg, int len)
{
	return sendto(sid, msg, len, 0, remote.getAddress(), sizeof(INADDR)) != SOCKET_ERROR;
}

UDPPacket * UDPSocket::poll()
{
	if (WSAWaitForMultipleEvents(1, &event, false, 0, true) == WSA_WAIT_EVENT_0)
	{
		int FromLen = sizeof(INADDR);
		BYTE msg[520];
		INADDR src;
		int len = recvfrom(sid, (char*)msg, 520, 0, src.getAddress(), &FromLen);

		if (len <= 0)
			return NULL;

		return new UDPPacket(src, msg, len);
	}

	return NULL;
}


//////// Addressing ////////

INADDR::INADDR(DWORD ip, WORD port)
{
	family		= AF_INET;
	this->ip	= ip;
	this->port	= htons(port);
	memset(padding, 0, 8);
}

INADDR::INADDR()
{
	memset(this, 0, sizeof(INADDR));

	family = AF_INET;
}

bool INADDR::operator==(INADDR other)
{
	return ((other.ip == ip) && (other.port == port));
}

bool INADDR::operator==(INADDR * other)
{
	return ((other->ip == ip) && (other->port == port));
}

void INADDR::operator=(INADDR other)
{
	this->ip	= other.ip;
	this->port	= other.port;
}

void INADDR::operator=(INADDR * other)
{
	this->ip	= other->ip;
	this->port	= other->port;
}

char * INADDR::getString()
{
	in_addr in;
	in.S_un.S_addr = ip;

	return inet_ntoa(in);
}

int INADDR::getPort()
{
	return htons(port);
}

sockaddr * INADDR::getAddress()
{
	return (sockaddr*)this;
}

void INADDR::set(DWORD ip, WORD port)
{
	this->ip	= ip;
	this->port	= htons(port);
}


//////// Error codes ////////

char * WSAGetErrorString(int code)
{
	switch (code)
	{
		case WSANOTINITIALISED:			return "WSANOTINITIALISED";
		case WSAENETDOWN:				return "WSAENETDOWN";
		case WSAEINPROGRESS:			return "WSAEINPROGRESS";
		case WSA_NOT_ENOUGH_MEMORY:		return "WSA_NOT_ENOUGH_MEMORY";
		case WSA_INVALID_HANDLE:		return "WSA_INVALID_HANDLE";
		case WSA_INVALID_PARAMETER:		return "WSA_INVALID_PARAMETER";
		case WSAEFAULT:					return "WSAEFAULT";
		case WSAEINTR:					return "WSAEINTR";
		case WSAEINVAL:					return "WSAEINVAL";
		case WSAEISCONN:				return "WSAEISCONN";
		case WSAENETRESET:				return "WSAENETRESET";
		case WSAENOTSOCK:				return "WSAENOTSOCK";
		case WSAEOPNOTSUPP:				return "WSAEOPNOTSUPP";
		case WSAESHUTDOWN:				return "WSAESHUTDOWN";
		case WSAEWOULDBLOCK:			return "WSAEWOULDBLOCK";
		case WSAEMSGSIZE:				return "WSAEMSGSIZE";
		case WSAETIMEDOUT:				return "WSAETIMEDOUT";
		case WSAECONNRESET:				return "WSAECONNRESET";
		case WSAENOTCONN:				return "WSAENOTCONN";
		case WSAEDISCON:				return "WSAEDISCON";
		case WSA_IO_PENDING:			return "WSA_IO_PENDING";
		case WSA_OPERATION_ABORTED:		return "WSA_OPERATION_ABORTED";
		default:						return "Unknown error";
	};
}


//////// Address resolution ////////

DWORD resolveHostname(char * name)
{
	DWORD ip = inet_addr(name);

	if (ip == SOCKET_ERROR)
	{
		hostent * host = gethostbyname(name);

		if (host)
			ip = *((DWORD*)*(host->h_addr_list));
	}

	return ip;
}
